@@ -36,6 +36,7 @@ typedef enum tool_type_t {
 
 	TOOL_WHOAMI	= TOOL_EXOP | 0x100U,
 	TOOL_PASSWD	= TOOL_EXOP | 0x200U,
+	TOOL_VC		= TOOL_EXOP | 0x400U,
 
 	TOOL_WRITE	= (TOOL_ADD|TOOL_DELETE|TOOL_MODIFY|TOOL_MODRDN),
 	TOOL_READ	= (TOOL_SEARCH|TOOL_COMPARE),
@@ -60,8 +61,6 @@ extern char		*prog;
 
 /* connection */
 extern char		*ldapuri;
-extern char		*ldaphost;
-extern int		ldapport;
 extern int		use_tls;
 extern int		protocol;
 extern int		version;
