@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2004-2018 The OpenLDAP Foundation.
+ * Copyright 2004-2021 The OpenLDAP Foundation.
  * Portions Copyright 2004-2005 by Howard Chu, Symas Corp.
  * All rights reserved.
  *
@@ -32,7 +32,7 @@
 #include <ac/errno.h>
 #include <ac/string.h>
 
-#include "config.h"
+#include "slap-config.h"
 
 #ifdef DO_KRB5
 #include <lber.h>
@@ -66,18 +66,14 @@ static ObjectClass *oc_krb5KDCEntry;
 
 #ifdef DO_SAMBA
 #ifdef HAVE_GNUTLS
-#include <nettle/des.h>
 #include <nettle/md4.h>
-typedef unsigned char DES_cblock[8];
 #elif HAVE_OPENSSL
-#include <openssl/des.h>
 #include <openssl/md4.h>
 #else
 #error Unsupported crypto backend.
 #endif
 #include "ldap_utf8.h"
 
-static AttributeDescription *ad_sambaLMPassword;
 static AttributeDescription *ad_sambaNTPassword;
 static AttributeDescription *ad_sambaPwdLastSet;
 static AttributeDescription *ad_sambaPwdMustChange;
@@ -135,29 +131,6 @@ static int smbk5pwd_modules_init( smbk5pwd_t *pi );
 #ifdef DO_SAMBA
 static const char hex[] = "0123456789abcdef";
 
-/* From liblutil/passwd.c... */
-static void lmPasswd_to_key(
-	const char *lmPasswd,
-	DES_cblock *key)
-{
-	const unsigned char *lpw = (const unsigned char *)lmPasswd;
-	unsigned char *k = (unsigned char *)key;
-
-	/* make room for parity bits */
-	k[0] = lpw[0];
-	k[1] = ((lpw[0]&0x01)<<7) | (lpw[1]>>1);
-	k[2] = ((lpw[1]&0x03)<<6) | (lpw[2]>>2);
-	k[3] = ((lpw[2]&0x07)<<5) | (lpw[3]>>3);
-	k[4] = ((lpw[3]&0x0F)<<4) | (lpw[4]>>4);
-	k[5] = ((lpw[4]&0x1F)<<3) | (lpw[5]>>5);
-	k[6] = ((lpw[5]&0x3F)<<2) | (lpw[6]>>6);
-	k[7] = ((lpw[6]&0x7F)<<1);
-
-#ifdef HAVE_OPENSSL
-	DES_set_odd_parity( key );
-#endif
-}
-
 #define MAX_PWLEN 256
 #define	HASHLEN	16
 
@@ -182,45 +155,6 @@ static void hexify(
 	*a++ = '\0';
 }
 
-static void lmhash(
-	struct berval *passwd,
-	struct berval *hash
-)
-{
-	char UcasePassword[15];
-	DES_cblock key;
-	DES_cblock StdText = "KGS!@#$%";
-	DES_cblock hbuf[2];
-#ifdef HAVE_OPENSSL
-	DES_key_schedule schedule;
-#elif defined(HAVE_GNUTLS)
-	struct des_ctx ctx;
-#endif
-
-	strncpy( UcasePassword, passwd->bv_val, 14 );
-	UcasePassword[14] = '\0';
-	ldap_pvt_str2upper( UcasePassword );
-
-	lmPasswd_to_key( UcasePassword, &key );
-#ifdef HAVE_GNUTLS
-	des_set_key( &ctx, key );
-	des_encrypt( &ctx, sizeof(key), hbuf[0], StdText );
-
-	lmPasswd_to_key( &UcasePassword[7], &key );
-	des_set_key( &ctx, key );
-	des_encrypt( &ctx, sizeof(key), hbuf[1], StdText );
-#elif defined(HAVE_OPENSSL)
-	DES_set_key_unchecked( &key, &schedule );
-	DES_ecb_encrypt( &StdText, &hbuf[0], &schedule , DES_ENCRYPT );
-
-	lmPasswd_to_key( &UcasePassword[7], &key );
-	DES_set_key_unchecked( &key, &schedule );
-	DES_ecb_encrypt( &StdText, &hbuf[1], &schedule , DES_ENCRYPT );
-#endif
-
-	hexify( (char *)hbuf, hash );
-}
-
 static void nthash(
 	struct berval *passwd,
 	struct berval *hash
@@ -355,7 +289,7 @@ static int k5key_chk(
 			struct lutil_tm tm;
 			struct lutil_timet tt;
 			if ( lutil_parsetime( a->a_vals[0].bv_val, &tm ) == 0 &&
-				lutil_tm2time( &tm, &tt ) == 0 && tt.tt_usec < op->o_time ) {
+				lutil_tm2time( &tm, &tt ) == 0 && tt.tt_sec < op->o_time ) {
 				/* Account is expired */
 				rc = LUTIL_PASSWD_ERR;
 				break;
@@ -460,7 +394,7 @@ static int smbk5pwd_exop_passwd(
 			/* shouldn't happen, this is a required attr */
 			Debug( LDAP_DEBUG_ANY, "%s smbk5pwd EXOP: "
 				"dn=\"%s\" missing krb5KeyVersionNumber\n",
-				op->o_log_prefix, e->e_name.bv_val, 0 );
+				op->o_log_prefix, e->e_name.bv_val );
 		}
 
 		ret = hdb_generate_key_set_password(context, ent.principal,
@@ -530,7 +464,7 @@ static int smbk5pwd_exop_passwd(
 		struct berval *keys;
 		ber_len_t j,l;
 		wchar_t *wcs, wc;
-		char *c, *d;
+		char *c;
 		struct berval pwd;
 		
 		/* Expand incoming UTF8 string to UCS4 */
@@ -568,33 +502,6 @@ static int smbk5pwd_exop_passwd(
 		ml->sml_values = keys;
 		ml->sml_nvalues = NULL;
 
-		/* Truncate UCS2 to 8-bit ASCII */
-		c = pwd.bv_val+1;
-		d = pwd.bv_val+2;
-		for (j=1; j<l; j++) {
-			*c++ = *d++;
-			d++;
-		}
-		pwd.bv_len /= 2;
-		pwd.bv_val[pwd.bv_len] = '\0';
-
-		ml = ch_malloc(sizeof(Modifications));
-		ml->sml_next = qpw->rs_mods;
-		qpw->rs_mods = ml;
-
-		keys = ch_malloc( 2 * sizeof(struct berval) );
-		BER_BVZERO( &keys[1] );
-		lmhash( &pwd, keys );
-		
-		ml->sml_desc = ad_sambaLMPassword;
-		ml->sml_op = LDAP_MOD_REPLACE;
-#ifdef SLAP_MOD_INTERNAL
-		ml->sml_flags = SLAP_MOD_INTERNAL;
-#endif
-		ml->sml_numvals = 1;
-		ml->sml_values = keys;
-		ml->sml_nvalues = NULL;
-
 		ch_free(wcs);
 
 		ml = ch_malloc(sizeof(Modifications));
@@ -719,16 +626,19 @@ static ConfigTable smbk5pwd_cfats[] = {
 		2, 0, 0, ARG_MAGIC|PC_SMB_ENABLE, smbk5pwd_cf_func,
 		"( OLcfgCtAt:1.1 NAME 'olcSmbK5PwdEnable' "
 		"DESC 'Modules to be enabled' "
+		"EQUALITY caseIgnoreMatch "
 		"SYNTAX OMsDirectoryString )", NULL, NULL },
 	{ "smbk5pwd-must-change", "time",
 		2, 2, 0, ARG_MAGIC|ARG_INT|PC_SMB_MUST_CHANGE, smbk5pwd_cf_func,
 		"( OLcfgCtAt:1.2 NAME 'olcSmbK5PwdMustChange' "
 		"DESC 'Credentials validity interval' "
+		"EQUALITY integerMatch "
 		"SYNTAX OMsInteger SINGLE-VALUE )", NULL, NULL },
 	{ "smbk5pwd-can-change", "time",
 		2, 2, 0, ARG_MAGIC|ARG_INT|PC_SMB_CAN_CHANGE, smbk5pwd_cf_func,
 		"( OLcfgCtAt:1.3 NAME 'olcSmbK5PwdCanChange' "
 		"DESC 'Credentials minimum validity interval' "
+		"EQUALITY integerMatch "
 		"SYNTAX OMsInteger SINGLE-VALUE )", NULL, NULL },
 
 	{ NULL, NULL, 0, 0, 0, ARG_IGNORED }
@@ -834,7 +744,7 @@ smbk5pwd_cf_func( ConfigArgs *c )
 		if ( c->value_int < 0 ) {
 			Debug( LDAP_DEBUG_ANY, "%s: smbk5pwd: "
 				"<%s> invalid negative value \"%d\".",
-				c->log, c->argv[ 0 ], 0 );
+				c->log, c->argv[ 0 ], c->value_int );
 			return 1;
 		}
 		pi->smb_must_change = c->value_int;
@@ -842,7 +752,7 @@ smbk5pwd_cf_func( ConfigArgs *c )
 		Debug( LDAP_DEBUG_ANY, "%s: smbk5pwd: "
 			"<%s> only meaningful "
 			"when compiled with -DDO_SAMBA.\n",
-			c->log, c->argv[ 0 ], 0 );
+			c->log, c->argv[ 0 ] );
 		return 1;
 #endif /* ! DO_SAMBA */
 		break;
@@ -852,7 +762,7 @@ smbk5pwd_cf_func( ConfigArgs *c )
                 if ( c->value_int < 0 ) {
                         Debug( LDAP_DEBUG_ANY, "%s: smbk5pwd: "
                                 "<%s> invalid negative value \"%d\".",
-                                c->log, c->argv[ 0 ], 0 );
+                                c->log, c->argv[ 0 ], c->value_int );
                         return 1;
                 }
                 pi->smb_can_change = c->value_int;
@@ -860,7 +770,7 @@ smbk5pwd_cf_func( ConfigArgs *c )
                 Debug( LDAP_DEBUG_ANY, "%s: smbk5pwd: "
                         "<%s> only meaningful "
                         "when compiled with -DDO_SAMBA.\n",
-                        c->log, c->argv[ 0 ], 0 );
+                        c->log, c->argv[ 0 ] );
                 return 1;
 #endif /* ! DO_SAMBA */
                 break;
@@ -946,7 +856,6 @@ smbk5pwd_modules_init( smbk5pwd_t *pi )
 #endif /* DO_KRB5 */
 #ifdef DO_SAMBA
 	samba_ad[] = {
-		{ "sambaLMPassword",		&ad_sambaLMPassword },
 		{ "sambaNTPassword",		&ad_sambaNTPassword },
 		{ "sambaPwdLastSet",		&ad_sambaPwdLastSet },
 		{ "sambaPwdMustChange",		&ad_sambaPwdMustChange },
@@ -976,8 +885,7 @@ smbk5pwd_modules_init( smbk5pwd_t *pi )
 		oc_krb5KDCEntry = oc_find( "krb5KDCEntry" );
 		if ( !oc_krb5KDCEntry ) {
 			Debug( LDAP_DEBUG_ANY, "smbk5pwd: "
-				"unable to find \"krb5KDCEntry\" objectClass.\n",
-				0, 0, 0 );
+				"unable to find \"krb5KDCEntry\" objectClass.\n" );
 			return -1;
 		}
 
@@ -1001,7 +909,7 @@ smbk5pwd_modules_init( smbk5pwd_t *pi )
 		if (ret) {
 			Debug( LDAP_DEBUG_ANY, "smbk5pwd: "
 				"unable to initialize krb5 context (%d).\n",
-				ret, 0, 0 );
+				ret );
 			oc_krb5KDCEntry = NULL;
 			return -1;
 		}
@@ -1018,7 +926,7 @@ smbk5pwd_modules_init( smbk5pwd_t *pi )
 				err_msg = (char *)krb5_get_err_text( context, ret );
 			Debug( LDAP_DEBUG_ANY, "smbk5pwd: "
 				"unable to initialize krb5 admin context: %s (%d).\n",
-				err_str ? err_str : err_msg, ret, 0 );
+				err_str ? err_str : err_msg, ret );
 			if (err_str)
 				krb5_free_error_string( context, err_str );
 			krb5_free_context( context );
@@ -1037,8 +945,7 @@ smbk5pwd_modules_init( smbk5pwd_t *pi )
 		oc_sambaSamAccount = oc_find( "sambaSamAccount" );
 		if ( !oc_sambaSamAccount ) {
 			Debug( LDAP_DEBUG_ANY, "smbk5pwd: "
-				"unable to find \"sambaSamAccount\" objectClass.\n",
-				0, 0, 0 );
+				"unable to find \"sambaSamAccount\" objectClass.\n" );
 			return -1;
 		}
 
@@ -1066,8 +973,7 @@ smbk5pwd_modules_init( smbk5pwd_t *pi )
 		oc_shadowAccount = oc_find( "shadowAccount" );
 		if ( !oc_shadowAccount ) {
 			Debug( LDAP_DEBUG_ANY, "smbk5pwd: "
-				"unable to find \"shadowAccount\" objectClass.\n",
-				0, 0, 0 );
+				"unable to find \"shadowAccount\" objectClass.\n" );
 			return -1;
 		}
 
@@ -1146,6 +1052,7 @@ smbk5pwd_initialize(void)
 
 	smbk5pwd.on_bi.bi_type = "smbk5pwd";
 
+	smbk5pwd.on_bi.bi_flags = SLAPO_BFLAG_SINGLE;
 	smbk5pwd.on_bi.bi_db_init = smbk5pwd_db_init;
 	smbk5pwd.on_bi.bi_db_open = smbk5pwd_db_open;
 	smbk5pwd.on_bi.bi_db_destroy = smbk5pwd_db_destroy;
