@@ -70,17 +70,12 @@ extern int h_errno;
 
 /* USE_GMTIME_R and USE_LOCALTIME_R defined in ldap_pvt.h */
 
-#ifdef LDAP_DEVEL
-	/* to be released with 2.5 */
 #if !defined( USE_GMTIME_R ) || !defined( USE_LOCALTIME_R )
 	/* we use the same mutex for gmtime(3) and localtime(3)
 	 * because implementations may use the same buffer
 	 * for both functions */
 	static ldap_pvt_thread_mutex_t ldap_int_gmtime_mutex;
 #endif
-#else /* ! LDAP_DEVEL */
-	ldap_pvt_thread_mutex_t ldap_int_gmtime_mutex;
-#endif /* ! LDAP_DEVEL */
 
 # if defined(HAVE_GETHOSTBYNAME_R) && \
 	(GETHOSTBYNAME_R_NARGS < 5) || (6 < GETHOSTBYNAME_R_NARGS)
@@ -183,7 +178,7 @@ static int _ldap_pvt_gt_subs;
 
 #ifdef _WIN32
 /* Windows SYSTEMTIME only has 10 millisecond resolution, so we
- * also need to use a high resolution timer to get microseconds.
+ * also need to use a high resolution timer to get nanoseconds.
  * This is pretty clunky.
  */
 static LARGE_INTEGER _ldap_pvt_gt_freq;
@@ -192,9 +187,10 @@ static int _ldap_pvt_gt_offset;
 
 #define SEC_TO_UNIX_EPOCH 11644473600LL
 #define TICKS_PER_SECOND 10000000
+#define BILLION	1000000000L
 
 static int
-ldap_pvt_gettimeusec(int *sec)
+ldap_pvt_gettimensec(int *sec)
 {
 	LARGE_INTEGER count;
 
@@ -205,7 +201,7 @@ ldap_pvt_gettimeusec(int *sec)
 	 */
 	LDAP_MUTEX_LOCK( &ldap_int_gettime_mutex );
 	/* We assume Windows has at least a vague idea of
-	 * when a second begins. So we align our microsecond count
+	 * when a second begins. So we align our nanosecond count
 	 * with the Windows millisecond count using this offset.
 	 * We retain the submillisecond portion of our own count.
 	 *
@@ -219,7 +215,7 @@ ldap_pvt_gettimeusec(int *sec)
 		ULARGE_INTEGER ut;
 		FILETIME ft0, ft1;
 		long long t;
-		int usec;
+		int nsec;
 
 		/* Initialize our offset */
 		QueryPerformanceFrequency( &_ldap_pvt_gt_freq );
@@ -237,13 +233,13 @@ ldap_pvt_gettimeusec(int *sec)
 		/* get second and fraction portion of counter */
 		t = c2.QuadPart % (_ldap_pvt_gt_freq.QuadPart*10);
 
-		/* convert to microseconds */
-		t *= 1000000;
-		usec = t / _ldap_pvt_gt_freq.QuadPart;
+		/* convert to nanoseconds */
+		t *= BILLION;
+		nsec = t / _ldap_pvt_gt_freq.QuadPart;
 
 		ut.QuadPart /= 10;
-		ut.QuadPart %= 10000000;
-		_ldap_pvt_gt_offset = usec - ut.QuadPart;
+		ut.QuadPart %= (10 * BILLION);
+		_ldap_pvt_gt_offset = nsec - ut.QuadPart;
 		count = c2;
 	}
 	if ( count.QuadPart <= _ldap_pvt_gt_prev.QuadPart ) {
@@ -254,28 +250,28 @@ ldap_pvt_gettimeusec(int *sec)
 	}
 	LDAP_MUTEX_UNLOCK( &ldap_int_gettime_mutex );
 
-	/* convert to microseconds */
+	/* convert to nanoseconds */
 	count.QuadPart %= _ldap_pvt_gt_freq.QuadPart*10;
-	count.QuadPart *= 1000000;
+	count.QuadPart *= BILLION;
 	count.QuadPart /= _ldap_pvt_gt_freq.QuadPart;
 	count.QuadPart -= _ldap_pvt_gt_offset;
 
-	/* We've extracted the 1s and microseconds.
-	 * The 1sec digit is used to detect wraparound in microsecnds.
+	/* We've extracted the 1s and nanoseconds.
+	 * The 1sec digit is used to detect wraparound in nanosecnds.
 	 */
 	if (count.QuadPart < 0)
-		count.QuadPart += 10000000;
-	else if (count.QuadPart >= 10000000)
-		count.QuadPart -= 10000000;
+		count.QuadPart += (10 * BILLION);
+	else if (count.QuadPart >= (10 * BILLION))
+		count.QuadPart -= (10 * BILLION);
 
-	*sec = count.QuadPart / 1000000;
-	return count.QuadPart % 1000000;
+	*sec = count.QuadPart / BILLION;
+	return count.QuadPart % BILLION;
 }
 
 
-/* emulate POSIX gettimeofday */
+/* emulate POSIX clock_gettime */
 int
-ldap_pvt_gettimeofday( struct timeval *tv, void *unused )
+ldap_pvt_clock_gettime( int clk_id, struct timespec *tv )
 {
 	FILETIME ft;
 	ULARGE_INTEGER ut;
@@ -285,11 +281,11 @@ ldap_pvt_gettimeofday( struct timeval *tv, void *unused )
 	ut.LowPart = ft.dwLowDateTime;
 	ut.HighPart = ft.dwHighDateTime;
 
-	/* convert to usec */
-	ut.QuadPart /= (TICKS_PER_SECOND / 1000000);
+	/* convert to sec */
+	ut.QuadPart /= TICKS_PER_SECOND;
 
-	tv->tv_usec = ldap_pvt_gettimeusec(&sec);
-	tv->tv_sec = ut.QuadPart / 1000000 - SEC_TO_UNIX_EPOCH;
+	tv->tv_nsec = ldap_pvt_gettimensec(&sec);
+	tv->tv_sec = ut.QuadPart - SEC_TO_UNIX_EPOCH;
 
 	/* check for carry from microseconds */
 	sec0 = tv->tv_sec % 10;
@@ -299,7 +295,19 @@ ldap_pvt_gettimeofday( struct timeval *tv, void *unused )
 	return 0;
 }
 
-/* return a broken out time, with microseconds
+/* emulate POSIX gettimeofday */
+int
+ldap_pvt_gettimeofday( struct timeval *tv, void *unused )
+{
+	struct timespec ts;
+	ldap_pvt_clock_gettime( 0, &ts );
+	tv->tv_sec = ts.tv_sec;
+	tv->tv_usec = ts.tv_nsec / 1000;
+	return 0;
+}
+
+
+/* return a broken out time, with nanoseconds
  */
 void
 ldap_pvt_gettime( struct lutil_tm *tm )
@@ -310,10 +318,10 @@ ldap_pvt_gettime( struct lutil_tm *tm )
 	31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
 
 	GetSystemTime( &st );
-	tm->tm_usec = ldap_pvt_gettimeusec(&sec);
+	tm->tm_nsec = ldap_pvt_gettimensec(&sec);
 	tm->tm_usub = _ldap_pvt_gt_subs;
 
-	/* any difference larger than microseconds is
+	/* any difference larger than nanoseconds is
 	 * already reflected in st
 	 */
 	tm->tm_sec = st.wSecond;
@@ -323,7 +331,7 @@ ldap_pvt_gettime( struct lutil_tm *tm )
 	tm->tm_mon = st.wMonth - 1;
 	tm->tm_year = st.wYear - 1900;
 
-	/* check for carry from microseconds */
+	/* check for carry from nanoseconds */
 	sec0 = tm->tm_sec % 10;
 	if (sec0 < sec || (sec0 == 9 && !sec)) {
 		tm->tm_sec++;
@@ -362,23 +370,36 @@ ldap_pvt_gettime( struct lutil_tm *tm )
 }
 #else
 
+#ifdef HAVE_CLOCK_GETTIME
+static struct timespec _ldap_pvt_gt_prevTv;
+#else
 static struct timeval _ldap_pvt_gt_prevTv;
+#endif
 
 void
 ldap_pvt_gettime( struct lutil_tm *ltm )
 {
-	struct timeval tv;
-
 	struct tm tm;
 	time_t t;
+#ifdef HAVE_CLOCK_GETTIME
+#define	FRAC	tv_nsec
+#define	NSECS(x)	x
+	struct timespec tv;
+
+	clock_gettime( CLOCK_REALTIME, &tv );
+#else
+#define	FRAC	tv_usec
+#define	NSECS(x)	x * 1000
+	struct timeval tv;
 
 	gettimeofday( &tv, NULL );
+#endif
 	t = tv.tv_sec;
 
 	LDAP_MUTEX_LOCK( &ldap_int_gettime_mutex );
 	if ( tv.tv_sec < _ldap_pvt_gt_prevTv.tv_sec
 		|| ( tv.tv_sec == _ldap_pvt_gt_prevTv.tv_sec
-		&& tv.tv_usec <= _ldap_pvt_gt_prevTv.tv_usec )) {
+		&& tv.FRAC <= _ldap_pvt_gt_prevTv.FRAC )) {
 		_ldap_pvt_gt_subs++;
 	} else {
 		_ldap_pvt_gt_subs = 0;
@@ -396,7 +417,7 @@ ldap_pvt_gettime( struct lutil_tm *ltm )
 	ltm->tm_mday = tm.tm_mday;
 	ltm->tm_mon = tm.tm_mon;
 	ltm->tm_year = tm.tm_year;
-	ltm->tm_usec = tv.tv_usec;
+	ltm->tm_nsec = NSECS(tv.FRAC);
 }
 #endif
 
@@ -411,7 +432,7 @@ ldap_pvt_csnstr(char *buf, size_t len, unsigned int replica, unsigned int mod)
 	n = snprintf( buf, len,
 		"%4d%02d%02d%02d%02d%02d.%06dZ#%06x#%03x#%06x",
 		tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour,
-		tm.tm_min, tm.tm_sec, tm.tm_usec, tm.tm_usub, replica, mod );
+		tm.tm_min, tm.tm_sec, tm.tm_nsec / 1000, tm.tm_usub, replica, mod );
 
 	if( n < 0 ) return 0;
 	return ( (size_t) n < len ) ? n : 0;
@@ -450,12 +471,16 @@ int ldap_pvt_gethostbyname_a(
 		*result=gethostbyname_r( name, resbuf, *buf, buflen, herrno_ptr );
 		r = (*result == NULL) ?  -1 : 0;
 #else
-		r = gethostbyname_r( name, resbuf, *buf,
-			buflen, result, herrno_ptr );
+		while((r = gethostbyname_r( name, resbuf, *buf, buflen, result, herrno_ptr )) == ERANGE) {
+			/* Increase the buffer */
+			buflen*=2;
+			if (safe_realloc(buf, buflen) == NULL)
+				return -1;
+		}
 #endif
 
-		Debug( LDAP_DEBUG_TRACE, "ldap_pvt_gethostbyname_a: host=%s, r=%d\n",
-		       name, r, 0 );
+		Debug2( LDAP_DEBUG_TRACE, "ldap_pvt_gethostbyname_a: host=%s, r=%d\n",
+		       name, r );
 
 #ifdef NETDB_INTERNAL
 		if ((r<0) &&
@@ -715,9 +740,6 @@ void ldap_int_utils_init( void )
 
 	ldap_pvt_thread_mutex_init( &ldap_int_gettime_mutex );
 
-#ifdef HAVE_GSSAPI
-	ldap_pvt_thread_mutex_init( &ldap_int_gssapi_mutex );
-#endif
 #endif
 
 	/* call other module init functions here... */
@@ -921,3 +943,84 @@ char *ldap_pvt_gai_strerror (int code) {
 	return _("Unknown error");
 }
 #endif
+
+/* format a socket address as a string */
+
+#ifdef HAVE_TCPD
+# include <tcpd.h>
+# define SOCKADDR_STRING_UNKNOWN	STRING_UNKNOWN
+#else /* ! TCP Wrappers */
+# define SOCKADDR_STRING_UNKNOWN	"unknown"
+#endif /* ! TCP Wrappers */
+
+void
+ldap_pvt_sockaddrstr( Sockaddr *sa, struct berval *addrbuf )
+{
+	char *addr;
+	switch( sa->sa_addr.sa_family ) {
+#ifdef LDAP_PF_LOCAL
+	case AF_LOCAL:
+		addrbuf->bv_len = snprintf( addrbuf->bv_val, addrbuf->bv_len,
+			"PATH=%s", sa->sa_un_addr.sun_path );
+		break;
+#endif
+#ifdef LDAP_PF_INET6
+	case AF_INET6:
+		strcpy(addrbuf->bv_val, "IP=");
+		if ( IN6_IS_ADDR_V4MAPPED(&sa->sa_in6_addr.sin6_addr) ) {
+#if defined( HAVE_GETADDRINFO ) && defined( HAVE_INET_NTOP )
+			addr = (char *)inet_ntop( AF_INET,
+			   ((struct in_addr *)&sa->sa_in6_addr.sin6_addr.s6_addr[12]),
+			   addrbuf->bv_val+3, addrbuf->bv_len-3 );
+#else /* ! HAVE_GETADDRINFO || ! HAVE_INET_NTOP */
+			addr = inet_ntoa( *((struct in_addr *)
+					&sa->sa_in6_addr.sin6_addr.s6_addr[12]) );
+#endif /* ! HAVE_GETADDRINFO || ! HAVE_INET_NTOP */
+			if ( !addr ) addr = SOCKADDR_STRING_UNKNOWN;
+			if ( addr != addrbuf->bv_val+3 ) {
+				addrbuf->bv_len = sprintf( addrbuf->bv_val+3, "%s:%d", addr,
+				 (unsigned) ntohs( sa->sa_in6_addr.sin6_port ) ) + 3;
+			} else {
+				int len = strlen( addr );
+				addrbuf->bv_len = sprintf( addr+len, ":%d",
+				 (unsigned) ntohs( sa->sa_in6_addr.sin6_port ) ) + len + 3;
+			}
+		} else {
+			addr = (char *)inet_ntop( AF_INET6,
+				      &sa->sa_in6_addr.sin6_addr,
+				      addrbuf->bv_val+4, addrbuf->bv_len-4 );
+			if ( !addr ) addr = SOCKADDR_STRING_UNKNOWN;
+			if ( addr != addrbuf->bv_val+4 ) {
+				addrbuf->bv_len = sprintf( addrbuf->bv_val+3, "[%s]:%d", addr,
+				 (unsigned) ntohs( sa->sa_in6_addr.sin6_port ) ) + 3;
+			} else {
+				int len = strlen( addr );
+				addrbuf->bv_val[3] = '[';
+				addrbuf->bv_len = sprintf( addr+len, "]:%d",
+				 (unsigned) ntohs( sa->sa_in6_addr.sin6_port ) ) + len + 4;
+			}
+		}
+		break;
+#endif /* LDAP_PF_INET6 */
+	case AF_INET:
+		strcpy(addrbuf->bv_val, "IP=");
+#if defined( HAVE_GETADDRINFO ) && defined( HAVE_INET_NTOP )
+		addr = (char *)inet_ntop( AF_INET, &sa->sa_in_addr.sin_addr,
+			   addrbuf->bv_val+3, addrbuf->bv_len-3 );
+#else /* ! HAVE_GETADDRINFO || ! HAVE_INET_NTOP */
+		addr = inet_ntoa( sa->sa_in_addr.sin_addr );
+#endif /* ! HAVE_GETADDRINFO || ! HAVE_INET_NTOP */
+		if ( !addr ) addr = SOCKADDR_STRING_UNKNOWN;
+		if ( addr != addrbuf->bv_val+3 ) {
+			addrbuf->bv_len = sprintf( addrbuf->bv_val+3, "%s:%d", addr,
+			 (unsigned) ntohs( sa->sa_in_addr.sin_port ) ) + 3;
+		} else {
+			int len = strlen( addr );
+			addrbuf->bv_len = sprintf( addr+len, ":%d",
+			 (unsigned) ntohs( sa->sa_in_addr.sin_port ) ) + len + 3;
+		}
+		break;
+	default:
+		addrbuf->bv_val[0] = '\0';
+	}
+}
