@@ -123,18 +123,18 @@ at_bvfind( struct berval *name )
 	struct aindexrec *air;
 
 	if ( attr_cache ) {
-		air = avl_find( attr_cache, name, attr_index_name_cmp );
+		air = ldap_avl_find( attr_cache, name, attr_index_name_cmp );
 		if ( air ) return air->air_at;
 	}
 
-	air = avl_find( attr_index, name, attr_index_name_cmp );
+	air = ldap_avl_find( attr_index, name, attr_index_name_cmp );
 
 	if ( air ) {
 		if ( air->air_at->sat_flags & SLAP_AT_DELETED ) {
 			air = NULL;
 		} else if (( slapMode & SLAP_TOOL_MODE ) && at_oc_cache ) {
-			avl_insert( &attr_cache, (caddr_t) air,
-				attr_index_cmp, avl_dup_error );
+			ldap_avl_insert( &attr_cache, (caddr_t) air,
+				attr_index_cmp, ldap_avl_dup_error );
 		}
 	}
 
@@ -200,7 +200,7 @@ at_delete_from_list(
 		list[i] = list[j];
 	}
 	list[i] = NULL;
-	/* Tell the runtime this can be shrinked */
+	/* Tell the runtime this can be shrunk */
 	list1 = ch_realloc(list, (i+1)*sizeof(AttributeType **));
 	if ( !list1 ) {
 		return -1;
@@ -239,7 +239,7 @@ at_delete_names( AttributeType *at )
 
 		ber_str2bv( *names, 0, 0, &tmpair.air_name );
 		tmpair.air_at = at;
-		air = (struct aindexrec *)avl_delete( &attr_index,
+		air = (struct aindexrec *)ldap_avl_delete( &attr_index,
 			(caddr_t)&tmpair, attr_index_cmp );
 		assert( air != NULL );
 		ldap_memfree( air );
@@ -325,7 +325,7 @@ at_destroy( void )
 		at_delete_names( a );
 	}
 
-	avl_free(attr_index, at_destroy_one);
+	ldap_avl_free(attr_index, at_destroy_one);
 
 	if ( slap_schema.si_at_undefined ) {
 		ad_destroy(slap_schema.si_at_undefined->sat_ad);
@@ -451,7 +451,7 @@ at_insert(
 		air->air_at = sat;
 		air_old = NULL;
 
-		if ( avl_insert( &attr_index, (caddr_t) air,
+		if ( ldap_avl_insert( &attr_index, (caddr_t) air,
 		                 attr_index_cmp, at_dup_error ) )
 		{
 			AttributeType	*old_sat;
@@ -515,8 +515,8 @@ at_insert(
 				ch_calloc( 1, sizeof(struct aindexrec) );
 			ber_str2bv( *names, 0, 0, &air->air_name );
 			air->air_at = sat;
-			if ( avl_insert( &attr_index, (caddr_t) air,
-			                 attr_index_cmp, avl_dup_error ) )
+			if ( ldap_avl_insert( &attr_index, (caddr_t) air,
+			                 attr_index_cmp, ldap_avl_dup_error ) )
 			{
 				AttributeType	*old_sat;
 				int		rc;
@@ -535,7 +535,7 @@ at_insert(
 					names--;
 					ber_str2bv( *names, 0, 0, &tmpair.air_name );
 					tmpair.air_at = sat;
-					air = (struct aindexrec *)avl_delete( &attr_index,
+					air = (struct aindexrec *)ldap_avl_delete( &attr_index,
 						(caddr_t)&tmpair, attr_index_cmp );
 					assert( air != NULL );
 					ldap_memfree( air );
@@ -546,7 +546,7 @@ at_insert(
 
 					ber_str2bv( sat->sat_oid, 0, 0, &tmpair.air_name );
 					tmpair.air_at = sat;
-					air = (struct aindexrec *)avl_delete( &attr_index,
+					air = (struct aindexrec *)ldap_avl_delete( &attr_index,
 						(caddr_t)&tmpair, attr_index_cmp );
 					assert( air != NULL );
 					ldap_memfree( air );
@@ -659,7 +659,7 @@ at_add(
 	*err = cname;
 
 	if ( !at->at_usage && at->at_no_user_mod ) {
-		/* user attribute must be modifable */
+		/* user attribute must be modifiable */
 		code = SLAP_SCHERR_ATTR_BAD_USAGE;
 		goto error_return;
 	}
@@ -725,7 +725,7 @@ at_add(
 	/*
 	 * Inherit definitions from superiors.  We only check the
 	 * direct superior since that one has already inherited from
-	 * its own superiorss
+	 * its own superiors
 	 */
 	if ( sat->sat_sup ) {
 		Syntax *syn = syn_find(sat->sat_sup->sat_syntax->ssyn_oid);
@@ -973,7 +973,7 @@ static void
 at_index_print( void )
 {
 	printf("Printing attribute type index:\n");
-	(void) avl_apply( attr_index, at_index_printnode, 0, -1, AVL_INORDER );
+	(void) ldap_avl_apply( attr_index, at_index_printnode, 0, -1, AVL_INORDER );
 }
 #endif
 #endif
@@ -1101,7 +1101,7 @@ register_at( const char *def, AttributeDescription **rad, int dupok )
 	}
 	if ( code ) {
 		Debug( LDAP_DEBUG_ANY, "register_at: AttributeType \"%s\": %s\n",
-			def, err, 0 );
+			def, err );
 	}
 	if ( rad ) *rad = ad;
 	return code;
