@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2001-2020 The OpenLDAP Foundation.
+ * Copyright 2001-2021 The OpenLDAP Foundation.
  * Portions Copyright 2001-2003 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -26,11 +26,11 @@
 
 #include <lutil.h>
 #include "slap.h"
-#include "config.h"
+#include "slap-config.h"
 #include "lber_pvt.h"
 #include "back-monitor.h"
 
-#include "config.h"
+#include "slap-config.h"
 
 #undef INTEGRATE_CORE_SCHEMA
 
@@ -68,7 +68,8 @@ static const monitor_extra_t monitor_extra = {
 
 	monitor_back_entry_stub,
 	monitor_back_entrypriv_create,
-	monitor_back_register_subsys_late
+	monitor_back_register_subsys_late,
+	monitor_back_entry_get_unlocked
 };
 	
 
@@ -322,12 +323,11 @@ monitor_back_register_subsys_late(
 	if ( be_monitor == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_register_subsys_late: "
-			"monitor database not configured.\n",
-			0, 0, 0 );
+			"monitor database not configured.\n" );
 		return -1;
 	}
 
-	/* everyting is ready, can register already */
+	/* everything is ready, can register already */
 	if ( monitor_subsys_is_opened() ) {
 		return monitor_back_register_subsys( ms );
 	}
@@ -384,8 +384,7 @@ monitor_back_register_database_limbo(
 	if ( be_monitor == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_register_database_limbo: "
-			"monitor database not configured.\n",
-			0, 0, 0 );
+			"monitor database not configured.\n" );
 		return -1;
 	}
 
@@ -429,8 +428,7 @@ monitor_back_register_overlay_limbo(
 	if ( be_monitor == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_register_overlay_limbo: "
-			"monitor database not configured.\n",
-			0, 0, 0 );
+			"monitor database not configured.\n" );
 		return -1;
 	}
 
@@ -464,12 +462,13 @@ monitor_back_register_entry(
 	unsigned long		flags )
 {
 	monitor_info_t 	*mi;
+	int rc = 0;
 
 	if ( be_monitor == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_register_entry(\"%s\"): "
 			"monitor database not configured.\n",
-			e->e_name.bv_val, 0, 0 );
+			e->e_name.bv_val );
 		return -1;
 	}
 
@@ -486,14 +485,13 @@ monitor_back_register_entry(
 		struct berval	pdn = BER_BVNULL;
 		monitor_entry_t *mp = NULL,
 				*mp_parent = NULL;
-		int		rc = 0;
 
 		if ( monitor_cache_get( mi, &e->e_nname, &e_parent ) == 0 ) {
 			/* entry exists */
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"entry exists\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			monitor_cache_release( mi, e_parent );
 			return -1;
 		}
@@ -504,7 +502,7 @@ monitor_back_register_entry(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"parent \"%s\" not found\n",
-				e->e_name.bv_val, pdn.bv_val, 0 );
+				e->e_name.bv_val, pdn.bv_val );
 			return -1;
 		}
 
@@ -516,7 +514,7 @@ monitor_back_register_entry(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"parent \"%s\" is volatile\n",
-				e->e_name.bv_val, e_parent->e_name.bv_val, 0 );
+				e->e_name.bv_val, e_parent->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -526,7 +524,7 @@ monitor_back_register_entry(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"monitor_entrypriv_create() failed\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -536,7 +534,7 @@ monitor_back_register_entry(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"entry_dup() failed\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -563,7 +561,7 @@ monitor_back_register_entry(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"unable to add entry\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -593,7 +591,7 @@ done:;
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"entry_dup() failed\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			return -1;
 		}
 		
@@ -617,7 +615,7 @@ done:;
 		**elpp = el;
 	}
 
-	return 0;
+	return rc;
 }
 
 int
@@ -653,8 +651,7 @@ monitor_back_register_entry_parent(
 		/* need a filter */
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_register_entry_parent(\"\"): "
-			"need a valid filter\n",
-			0, 0, 0 );
+			"need a valid filter\n" );
 		return -1;
 	}
 
@@ -685,7 +682,7 @@ monitor_back_register_entry_parent(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry_parent(\"%s\"): "
 				"parent entry does not exist\n",
-				ndn.bv_val, 0, 0 );
+				ndn.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -698,7 +695,7 @@ monitor_back_register_entry_parent(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry_parent(\"%s\"): "
 				"entry is volatile\n",
-				e_parent->e_name.bv_val, 0, 0 );
+				e_parent->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -711,7 +708,7 @@ monitor_back_register_entry_parent(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry_parent(\"%s\"): "
 				"entry already exists\n",
-				e_name.bv_val, 0, 0 );
+				e_name.bv_val );
 			monitor_cache_release( mi, e_new );
 			e_new = NULL;
 			rc = -1;
@@ -723,7 +720,7 @@ monitor_back_register_entry_parent(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry_parent(\"%s\"): "
 				"monitor_entrypriv_create() failed\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -733,7 +730,7 @@ monitor_back_register_entry_parent(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"entry_dup() failed\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -764,7 +761,7 @@ monitor_back_register_entry_parent(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"unable to add entry\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -798,7 +795,7 @@ done:;
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"entry_dup() failed\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			goto done_limbo;
 		}
 		
@@ -992,16 +989,11 @@ monitor_back_register_entry_attrs(
 	if ( filter == NULL ) filter = &empty_bv;
 
 	if ( be_monitor == NULL ) {
-		char		buf[ SLAP_TEXT_BUFLEN ];
-
-		snprintf( buf, sizeof( buf ),
-			"monitor_back_register_entry_%s(base=\"%s\" scope=%s filter=\"%s\"): "
-			"monitor database not configured.\n",
-			fname,
-			BER_BVISNULL( nbase ) ? "" : nbase->bv_val,
-			ldap_pvt_scope2str( scope ),
-			BER_BVISNULL( filter ) ? "" : filter->bv_val );
-		Debug( LDAP_DEBUG_ANY, "%s\n", buf, 0, 0 );
+		Debug(LDAP_DEBUG_ANY,
+		      "monitor_back_register_entry_%s(base=\"%s\" scope=%s filter=\"%s\"): " "monitor database not configured.\n\n",
+		      fname, BER_BVISNULL(nbase) ? "" : nbase->bv_val,
+		      ldap_pvt_scope2str(scope),
+		      BER_BVISNULL(filter) ? "" : filter->bv_val );
 
 		return -1;
 	}
@@ -1026,7 +1018,7 @@ monitor_back_register_entry_attrs(
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_register_entry_%s(\"\"): "
 			"need a valid filter\n",
-			fname, 0, 0 );
+			fname );
 		return -1;
 	}
 
@@ -1040,19 +1032,12 @@ monitor_back_register_entry_attrs(
 
 		if ( BER_BVISNULL( &ndn ) ) {
 			if ( monitor_search2ndn( nbase, scope, filter, &ndn ) ) {
-				char		buf[ SLAP_TEXT_BUFLEN ];
-
-				snprintf( buf, sizeof( buf ),
-					"monitor_back_register_entry_%s(\"\"): "
-					"base=\"%s\" scope=%s filter=\"%s\": "
-					"unable to find entry\n",
-					fname,
-					nbase->bv_val ? nbase->bv_val : "\"\"",
-					ldap_pvt_scope2str( scope ),
-					filter->bv_val );
-
-				/* entry does not exist */
-				Debug( LDAP_DEBUG_ANY, "%s\n", buf, 0, 0 );
+				Debug(LDAP_DEBUG_ANY,
+				      "monitor_back_register_entry_%s(\"\"): " "base=\"%s\" scope=%s filter=\"%s\": " "unable to find entry\n\n",
+				      fname,
+				      nbase->bv_val ? nbase->bv_val : "\"\"",
+				      ldap_pvt_scope2str(scope),
+				      filter->bv_val );
 				return -1;
 			}
 
@@ -1064,7 +1049,7 @@ monitor_back_register_entry_attrs(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry_%s(\"%s\"): "
 				"entry does not exist\n",
-				fname, ndn.bv_val, 0 );
+				fname, ndn.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -1077,7 +1062,7 @@ monitor_back_register_entry_attrs(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry_%s(\"%s\"): "
 				"entry is volatile\n",
-				fname, e->e_name.bv_val, 0 );
+				fname, e->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -1100,7 +1085,7 @@ monitor_back_register_entry_attrs(
 						Debug( LDAP_DEBUG_ANY,
 							"monitor_back_register_entry_%s(\"%s\"): "
 							"attr_dup() failed\n",
-							fname, e->e_name.bv_val, 0 );
+							fname, e->e_name.bv_val );
 						rc = -1;
 						goto done;
 					}
@@ -1203,7 +1188,7 @@ monitor_back_unregister_entry(
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_unregister_entry(\"%s\"): "
 			"monitor database not configured.\n",
-			ndn->bv_val, 0, 0 );
+			ndn->bv_val );
 
 		return -1;
 	}
@@ -1228,7 +1213,7 @@ monitor_back_unregister_entry(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_unregister_entry(\"%s\"): "
 				"entry removal failed.\n",
-				ndn->bv_val, 0, 0 );
+				ndn->bv_val );
 			return -1;
 		}
 
@@ -1329,8 +1314,7 @@ monitor_back_unregister_entry_parent(
 		/* need a filter */
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_unregister_entry_parent(\"\"): "
-			"need a valid filter\n",
-			0, 0, 0 );
+			"need a valid filter\n" );
 		return -1;
 	}
 
@@ -1355,7 +1339,7 @@ monitor_back_unregister_entry_parent(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_unregister_entry(\"%s\"): "
 				"entry removal failed.\n",
-				ndn.bv_val, 0, 0 );
+				ndn.bv_val );
 			ber_memfree( ndn.bv_val );
 			return -1;
 		}
@@ -1448,16 +1432,11 @@ monitor_back_unregister_entry_attrs(
 	char		*fname = ( target_a == NULL ? "callback" : "attrs" );
 
 	if ( be_monitor == NULL ) {
-		char		buf[ SLAP_TEXT_BUFLEN ];
-
-		snprintf( buf, sizeof( buf ),
-			"monitor_back_unregister_entry_%s(base=\"%s\" scope=%s filter=\"%s\"): "
-			"monitor database not configured.\n",
-			fname,
-			BER_BVISNULL( nbase ) ? "" : nbase->bv_val,
-			ldap_pvt_scope2str( scope ),
-			BER_BVISNULL( filter ) ? "" : filter->bv_val );
-		Debug( LDAP_DEBUG_ANY, "%s\n", buf, 0, 0 );
+		Debug(LDAP_DEBUG_ANY,
+		      "monitor_back_unregister_entry_%s(base=\"%s\" scope=%s filter=\"%s\"): " "monitor database not configured.\n\n",
+		      fname, BER_BVISNULL(nbase) ? "" : nbase->bv_val,
+		      ldap_pvt_scope2str(scope),
+		      BER_BVISNULL(filter) ? "" : filter->bv_val );
 
 		return -1;
 	}
@@ -1488,7 +1467,7 @@ monitor_back_unregister_entry_attrs(
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_unregister_entry_%s(\"\"): "
 			"need a valid filter\n",
-			fname, 0, 0 );
+			fname );
 		return -1;
 	}
 
@@ -1499,18 +1478,11 @@ monitor_back_unregister_entry_attrs(
 
 		if ( BER_BVISNULL( &ndn ) ) {
 			if ( monitor_search2ndn( nbase, scope, filter, &ndn ) ) {
-				char		buf[ SLAP_TEXT_BUFLEN ];
-
-				snprintf( buf, sizeof( buf ),
-					"monitor_back_unregister_entry_%s(\"\"): "
-					"base=\"%s\" scope=%d filter=\"%s\": "
-					"unable to find entry\n",
-					fname,
-					nbase->bv_val ? nbase->bv_val : "\"\"",
-					scope, filter->bv_val );
-
-				/* entry does not exist */
-				Debug( LDAP_DEBUG_ANY, "%s\n", buf, 0, 0 );
+				Debug(LDAP_DEBUG_ANY,
+				      "monitor_back_unregister_entry_%s(\"\"): " "base=\"%s\" scope=%d filter=\"%s\": " "unable to find entry\n\n",
+				      fname,
+				      nbase->bv_val ? nbase->bv_val : "\"\"",
+				      scope, filter->bv_val );
 				return -1;
 			}
 
@@ -1522,7 +1494,7 @@ monitor_back_unregister_entry_attrs(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_unregister_entry(\"%s\"): "
 				"entry removal failed.\n",
-				ndn.bv_val, 0, 0 );
+				ndn.bv_val );
 			return -1;
 		}
 
@@ -1990,9 +1962,11 @@ monitor_back_initialize(
 		{ "olmSubSystemAttributes",		"olmAttributes:0" },
 		{ "olmGenericAttributes",		"olmSubSystemAttributes:0" },
 		{ "olmDatabaseAttributes",		"olmSubSystemAttributes:1" },
+		{ "olmOverlayAttributes",		"olmSubSystemAttributes:2" },
+		{ "olmModuleAttributes",		"olmSubSystemAttributes:3" },
 
-		/* for example, back-bdb specific attrs
-		 * are in "olmDatabaseAttributes:1"
+		/* for example, back-mdb specific attrs
+		 * are in "olmDatabaseAttributes:12"
 		 *
 		 * NOTE: developers, please record here OID assignments
 		 * for other modules */
@@ -2001,9 +1975,11 @@ monitor_back_initialize(
 		{ "olmSubSystemObjectClasses",		"olmObjectClasses:0" },
 		{ "olmGenericObjectClasses",		"olmSubSystemObjectClasses:0" },
 		{ "olmDatabaseObjectClasses",		"olmSubSystemObjectClasses:1" },
+		{ "olmOverlayObjectClasses",		"olmSubSystemObjectClasses:2" },
+		{ "olmModuleObjectClasses",			"olmSubSystemObjectClasses:3" },
 
-		/* for example, back-bdb specific objectClasses
-		 * are in "olmDatabaseObjectClasses:1"
+		/* for example, back-mdb specific objectClasses
+		 * are in "olmDatabaseObjectClasses:12"
 		 *
 		 * NOTE: developers, please record here OID assignments
 		 * for other modules */
@@ -2029,7 +2005,7 @@ monitor_back_initialize(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_initialize: unable to add "
 				"objectIdentifier \"%s=%s\"\n",
-				s_oid[ i ].name, s_oid[ i ].oid, 0 );
+				s_oid[ i ].name, s_oid[ i ].oid );
 			return 1;
 		}
 	}
@@ -2045,7 +2021,7 @@ monitor_back_initialize(
 
 		if ( code ) {
 			Debug( LDAP_DEBUG_ANY,
-				"monitor_back_db_init: register_at failed\n", 0, 0, 0 );
+				"monitor_back_db_init: register_at failed\n" );
 			return -1;
 		}
 		(*ad)->ad_type->sat_flags |= mat[ i ].flags;
@@ -2059,7 +2035,7 @@ monitor_back_initialize(
 		code = register_oc( moc[ i ].schema, Oc, 0 );
 		if ( code ) {
 			Debug( LDAP_DEBUG_ANY,
-				"monitor_back_db_init: register_oc failed\n", 0, 0, 0 );
+				"monitor_back_db_init: register_oc failed\n" );
 			return -1;
 		}
 		(*Oc)->soc_flags |= moc[ i ].flags;
@@ -2170,7 +2146,7 @@ monitor_back_db_init(
 	if( rc != LDAP_SUCCESS ) {
 		Debug( LDAP_DEBUG_ANY,
 			"unable to normalize/pretty monitor DN \"%s\" (%d)\n",
-			dn.bv_val, rc, 0 );
+			dn.bv_val, rc );
 		return -1;
 	}
 
@@ -2292,7 +2268,7 @@ monitor_back_db_open(
 	if ( e == NULL) {
 		Debug( LDAP_DEBUG_ANY,
 			"unable to create \"%s\" entry\n",
-			SLAPD_MONITOR_DN, 0, 0 );
+			SLAPD_MONITOR_DN );
 		return( -1 );
 	}
 
@@ -2325,7 +2301,7 @@ monitor_back_db_open(
 					&bv, NULL ) ) {
 			Debug( LDAP_DEBUG_ANY,
 				"unable to add monitoredInfo to \"%s\" entry\n",
-				SLAPD_MONITOR_DN, 0, 0 );
+				SLAPD_MONITOR_DN );
 			return( -1 );
 		}
 	}
@@ -2340,7 +2316,7 @@ monitor_back_db_open(
 	if ( monitor_cache_add( mi, e ) ) {
 		Debug( LDAP_DEBUG_ANY,
 			"unable to add entry \"%s\" to cache\n",
-			SLAPD_MONITOR_DN, 0, 0 );
+			SLAPD_MONITOR_DN );
 		return -1;
 	}
 	root = e;
@@ -2362,7 +2338,7 @@ monitor_back_db_open(
 		if ( rc != LDAP_SUCCESS ) {
 			Debug( LDAP_DEBUG_ANY,
 				"monitor RDN \"%s\" is invalid\n", 
-				dn.bv_val, 0, 0 );
+				dn.bv_val );
 			return( -1 );
 		}
 
@@ -2373,7 +2349,7 @@ monitor_back_db_open(
 		if ( e == NULL) {
 			Debug( LDAP_DEBUG_ANY,
 				"unable to create \"%s\" entry\n", 
-				monitor_subsys[ i ]->mss_dn.bv_val, 0, 0 );
+				monitor_subsys[ i ]->mss_dn.bv_val );
 			return( -1 );
 		}
 		monitor_subsys[i]->mss_dn = e->e_name;
@@ -2395,7 +2371,7 @@ monitor_back_db_open(
 		if ( monitor_cache_add( mi, e ) ) {
 			Debug( LDAP_DEBUG_ANY,
 				"unable to add entry \"%s\" to cache\n",
-				monitor_subsys[ i ]->mss_dn.bv_val, 0, 0 );
+				monitor_subsys[ i ]->mss_dn.bv_val );
 			return -1;
 		}
 
@@ -2588,11 +2564,3 @@ monitor_back_db_destroy(
 
 	return 0;
 }
-
-#if SLAPD_MONITOR == SLAPD_MOD_DYNAMIC
-
-/* conditionally define the init_module() function */
-SLAP_BACKEND_INIT_MODULE( monitor )
-
-#endif /* SLAPD_MONITOR == SLAPD_MOD_DYNAMIC */
-
