@@ -30,8 +30,8 @@ starttls_extop ( Operation *op, SlapReply *rs )
 {
 	int rc;
 
-	Statslog( LDAP_DEBUG_STATS, "%s STARTTLS\n",
-	    op->o_log_prefix, 0, 0, 0, 0 );
+	Debug( LDAP_DEBUG_STATS, "%s STARTTLS\n",
+	    op->o_log_prefix );
 
 	if ( op->ore_reqdata != NULL ) {
 		/* no request data should be provided */
@@ -63,9 +63,9 @@ starttls_extop ( Operation *op, SlapReply *rs )
 	if ( !( global_disallows & SLAP_DISALLOW_TLS_2_ANON ) &&
 		( op->o_conn->c_dn.bv_len != 0 ) )
 	{
-		Statslog( LDAP_DEBUG_STATS,
+		Debug( LDAP_DEBUG_STATS,
 			"%s AUTHZ anonymous mech=starttls ssf=0\n",
-			op->o_log_prefix, 0, 0, 0, 0 );
+			op->o_log_prefix );
 
 		/* force to anonymous */
 		connection2anonymous( op->o_conn );
