@@ -1,7 +1,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2011 The OpenLDAP Foundation.
+ * Copyright 1999-2021 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,12 +40,18 @@
 #include "lber_pvt.h"
 #include "slapd-common.h"
 
-#define SEARCHCMD		"slapd-search"
-#define READCMD			"slapd-read"
-#define ADDCMD			"slapd-addel"
-#define MODRDNCMD		"slapd-modrdn"
-#define MODIFYCMD		"slapd-modify"
-#define BINDCMD			"slapd-bind"
+#ifdef _WIN32
+#define EXE		".exe"
+#else
+#define EXE
+#endif
+
+#define SEARCHCMD		"slapd-search" EXE
+#define READCMD			"slapd-read" EXE
+#define ADDCMD			"slapd-addel" EXE
+#define MODRDNCMD		"slapd-modrdn" EXE
+#define MODIFYCMD		"slapd-modify" EXE
+#define BINDCMD			"slapd-bind" EXE
 #define MAXARGS      		100
 #define MAXREQS			5000
 #define LOOPS			100
@@ -86,7 +92,7 @@ usage( char *name, char opt )
 
 	fprintf( stderr,
 		"usage: %s "
-		"-H <uri> | ([-h <host>] -p <port>) "
+		"-H <uri> "
 		"-D <manager> "
 		"-w <passwd> "
 		"-d <datadir> "
@@ -110,8 +116,6 @@ main( int argc, char **argv )
 {
 	int		i, j;
 	char		*uri = NULL;
-	char		*host = "localhost";
-	char		*port = NULL;
 	char		*manager = NULL;
 	char		*passwd = NULL;
 	char		*dirname = NULL;
@@ -245,7 +249,7 @@ main( int argc, char **argv )
 			break;
 
 		case 'd':		/* data directory */
-			dirname = strdup( optarg );
+			dirname = optarg;
 			break;
 
 		case 'F':
@@ -253,11 +257,7 @@ main( int argc, char **argv )
 			break;
 
 		case 'H':		/* slapd uri */
-			uri = strdup( optarg );
-			break;
-
-		case 'h':		/* slapd host */
-			host = strdup( optarg );
+			uri = optarg;
 			break;
 
 		case 'I':
@@ -319,7 +319,7 @@ main( int argc, char **argv )
 			break;
 
 		case 'L':		/* the number of outerloops per client */
-			outerloops = strdup( optarg );
+			outerloops = optarg;
 			break;
 
 		case 'N':
@@ -327,15 +327,11 @@ main( int argc, char **argv )
 			break;
 
 		case 'P':		/* prog directory */
-			progdir = strdup( optarg );
-			break;
-
-		case 'p':		/* the servers port number */
-			port = strdup( optarg );
+			progdir = optarg;
 			break;
 
 		case 'r':		/* the number of retries in case of error */
-			retries = strdup( optarg );
+			retries = optarg;
 			break;
 
 		case 'S':
@@ -343,7 +339,7 @@ main( int argc, char **argv )
 			break;
 
 		case 't':		/* the delay in seconds between each retry */
-			delay = strdup( optarg );
+			delay = optarg;
 			break;
 
 		case 'w':		/* the managers passwd */
@@ -365,7 +361,7 @@ main( int argc, char **argv )
 		}
 	}
 
-	if (( dirname == NULL ) || ( port == NULL && uri == NULL ) ||
+	if (( dirname == NULL ) || ( uri == NULL ) ||
 			( manager == NULL ) || ( passwd == NULL ) || ( progdir == NULL ))
 	{
 		usage( argv[0], '\0' );
@@ -521,15 +517,8 @@ main( int argc, char **argv )
 	snprintf( scmd, sizeof scmd, "%s" LDAP_DIRSEP SEARCHCMD,
 		progdir );
 	sargs[sanum++] = scmd;
-	if ( uri ) {
-		sargs[sanum++] = "-H";
-		sargs[sanum++] = uri;
-	} else {
-		sargs[sanum++] = "-h";
-		sargs[sanum++] = host;
-		sargs[sanum++] = "-p";
-		sargs[sanum++] = port;
-	}
+	sargs[sanum++] = "-H";
+	sargs[sanum++] = uri;
 	sargs[sanum++] = "-D";
 	sargs[sanum++] = manager;
 	sargs[sanum++] = "-w";
@@ -582,15 +571,8 @@ main( int argc, char **argv )
 	snprintf( rcmd, sizeof rcmd, "%s" LDAP_DIRSEP READCMD,
 		progdir );
 	rargs[ranum++] = rcmd;
-	if ( uri ) {
-		rargs[ranum++] = "-H";
-		rargs[ranum++] = uri;
-	} else {
-		rargs[ranum++] = "-h";
-		rargs[ranum++] = host;
-		rargs[ranum++] = "-p";
-		rargs[ranum++] = port;
-	}
+	rargs[ranum++] = "-H";
+	rargs[ranum++] = uri;
 	rargs[ranum++] = "-D";
 	rargs[ranum++] = manager;
 	rargs[ranum++] = "-w";
@@ -636,15 +618,8 @@ main( int argc, char **argv )
 	snprintf( ncmd, sizeof ncmd, "%s" LDAP_DIRSEP MODRDNCMD,
 		progdir );
 	nargs[nanum++] = ncmd;
-	if ( uri ) {
-		nargs[nanum++] = "-H";
-		nargs[nanum++] = uri;
-	} else {
-		nargs[nanum++] = "-h";
-		nargs[nanum++] = host;
-		nargs[nanum++] = "-p";
-		nargs[nanum++] = port;
-	}
+	nargs[nanum++] = "-H";
+	nargs[nanum++] = uri;
 	nargs[nanum++] = "-D";
 	nargs[nanum++] = manager;
 	nargs[nanum++] = "-w";
@@ -679,15 +654,8 @@ main( int argc, char **argv )
 	snprintf( mcmd, sizeof mcmd, "%s" LDAP_DIRSEP MODIFYCMD,
 		progdir );
 	margs[manum++] = mcmd;
-	if ( uri ) {
-		margs[manum++] = "-H";
-		margs[manum++] = uri;
-	} else {
-		margs[manum++] = "-h";
-		margs[manum++] = host;
-		margs[manum++] = "-p";
-		margs[manum++] = port;
-	}
+	margs[manum++] = "-H";
+	margs[manum++] = uri;
 	margs[manum++] = "-D";
 	margs[manum++] = manager;
 	margs[manum++] = "-w";
@@ -724,15 +692,8 @@ main( int argc, char **argv )
 	snprintf( acmd, sizeof acmd, "%s" LDAP_DIRSEP ADDCMD,
 		progdir );
 	aargs[aanum++] = acmd;
-	if ( uri ) {
-		aargs[aanum++] = "-H";
-		aargs[aanum++] = uri;
-	} else {
-		aargs[aanum++] = "-h";
-		aargs[aanum++] = host;
-		aargs[aanum++] = "-p";
-		aargs[aanum++] = port;
-	}
+	aargs[aanum++] = "-H";
+	aargs[aanum++] = uri;
 	aargs[aanum++] = "-D";
 	aargs[aanum++] = manager;
 	aargs[aanum++] = "-w";
@@ -770,25 +731,16 @@ main( int argc, char **argv )
 	if ( !noinit ) {
 		bargs[banum++] = "-I";	/* init on each bind */
 	}
-	if ( uri ) {
-		bargs[banum++] = "-H";
-		bargs[banum++] = uri;
-	} else {
-		bargs[banum++] = "-h";
-		bargs[banum++] = host;
-		bargs[banum++] = "-p";
-		bargs[banum++] = port;
-	}
+	bargs[banum++] = "-H";
+	bargs[banum++] = uri;
 	bargs[banum++] = "-l";
 	bargs[banum++] = bloops;
 	bargs[banum++] = "-L";
 	bargs[banum++] = outerloops;
-#if 0
 	bargs[banum++] = "-r";
 	bargs[banum++] = retries;
 	bargs[banum++] = "-t";
 	bargs[banum++] = delay;
-#endif
 	if ( friendly ) {
 		bargs[banum++] = friendlyOpt;
 	}
